@@ -1,6 +1,6 @@
 # frozen_string_literal: true
 
-RSpec.describe ProcessExecuter::DestinationBase do
+RSpec.describe ProcessExecuter::Destinations::DestinationBase do
   describe '.handles?' do
     it 'raises NotImplementedError' do
       expect { described_class.handles?('destination') }.to raise_error(NotImplementedError)
